const Joi = require('joi');
const client = require('../core/client');
const { getCurrencyISOCodes } = require('../core/utils');
const { validationError, failure } = require('../core/result');

const INVOICES_PATH = '/api/invoices';

const invoiceQueryParamSchema = Joi.object()
  .keys({
    currency: Joi.string()
      .valid(...getCurrencyISOCodes)
      .optional(),
    end_date: Joi.date().iso().optional(),
    payment_status: Joi.string().valid('PENDING', 'PAID').optional(),
    shop: Joi.number().positive().optional(),
    start_date: Joi.date().iso().optional(),
    states: Joi.array()
      .items(Joi.string().valid('DRAFT', 'GENERATED', 'COMPLETE'))
      .optional()
      .default(['GENERATED', 'COMPLETE']),
    type: Joi.string()
      .valid('ALL', 'AUTO_INVOICE', 'MANUAL_INVOICE', 'MANUAL_CREDIT')
      .optional()
      .default('AUTO_INVOICE'),
    max: Joi.number().positive().max(100).optional()
      .default(10),
    offset: Joi.number().min(0).optional().default(0),
    sort: Joi.string().optional(),
    order: Joi.string().valid('asc', 'desc').optional().default('asc'),
  })
  .optional();

const createInvoicesSchema = Joi.object()
  .keys({
    manual_accounting_documents: Joi.array()
      .items(
        Joi.object().keys({
          emission_date: Joi.date().iso().optional(),
          issued: Joi.bool().required(),
          shop_id: Joi.number().positive().required(),
          type: Joi.string()
            .valid('INVOICE', 'CREDIT').required(),
          lines: Joi.array()
            .items({
              amount: Joi.number().min(0).required(),
              description: Joi.string().required(),
              quantity: Joi.number().integer().min(0).required(),
              tax_codes: Joi.array().items(Joi.string().valid('TAXZERO', 'TAXDEFAULT')).length(1).required(),
            }).min(1).max(5000)
            .required()
        })
      ),
  }).required();

/**
 * List accounting documents [IV01]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#IV01}
 * Note: this resource supports offset pagination [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#offset-pagination}
 * Recommended: Once per day and as many times as there are invoice pages returned
 * Maximum: Once per day and as many times as there are invoice pages returned
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.params {@link https://help.mirakl.net/help/api-doc/operator/mmp.html#IV01}
 */
const getListInvoices = ({ url, headers, params }) => {
  const { error, value } = invoiceQueryParamSchema.validate(params);
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + INVOICES_PATH,
    headers,
    params: value ? { ...value, states: value.states.join(',') } : value,
  });
};
/**
 * Create accounting documents [IV03]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#IV03}
 * Create a manual accounting document
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.data {@link https://help.mirakl.net/help/api-doc/operator/mmp.html#IV03}
 */
const createInvoices = ({ url, headers, data }) => {
  const { error, value } = createInvoicesSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }

  return client.postRequest({
    url: url + INVOICES_PATH,
    headers,
    data: value,
  })
    .then((res) => {
      if (res.isOk
            && res.data.manual_accounting_document_returns[0]
            && res.data.manual_accounting_document_returns[0].manual_accounting_document_error
            && res.data.manual_accounting_document_returns[0].manual_accounting_document_error.errors
            && res.data.manual_accounting_document_returns[0].manual_accounting_document_error.errors.length) {
        return failure(res.data.manual_accounting_document_returns[0].manual_accounting_document_error.errors[0].message, res.data);
      }
      return res;
    });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient') {
      ctx.prototype.getListInvoices = function (params) {
        return getListInvoices({ ...this._config, params });
      };
      ctx.prototype.createInvoices = function (data) {
        return createInvoices({ ...this._config, data });
      };
    }
  }
};
