const Joi = require('joi');
const FormData = require('form-data');
const client = require('../core/client');
const { validationError, failure } = require('../core/result');

const MESSAGES_PATH = '/api/inbox/threads';
const MESSAGES_ORDERS_PATH = (id) => `/api/orders/${id}/threads`;

const seekPaginationSchema = {
  previous_page_token: Joi.string().optional(),
  next_page_token: Joi.string().optional(),
  limit: Joi.number().positive().max(100).optional(),
};

const createThreadSchema = Joi.object()
  .keys({
    files: Joi.array()
      .items(Joi.object({ pipe: Joi.func().required() }).unknown().required())
      .optional(),
    thread_input: Joi.object()
      .keys({
        body: Joi.string().min(3).max(50000).required(),
        to: Joi.array()
          .items(Joi.string().valid('OPERATOR', 'SHOP', 'CUSTOMER').required())
          .max(3)
          .required(),
        topic: Joi.object()
          .keys({
            type: Joi.string().valid('FREE_TEXT', 'REASON_CODE').required(),
            value: Joi.string().optional(),
          })
          .required(),
      })
      .required(),
  })
  .required();

/**
 * List all threads [M11]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#M11}
 * You may want to retrieve the threads linked to a specific entity, using both entity_type and entity_id.
 * For example, to retrieve threads for an order, use entity_type=MMP_ORDER&entity_id=my-order-1.
 * Available values for entity_type are:
 * - MMP_ORDER: for threads on product orders
 * - MMP_OFFER: for threads on offers
 * - MPS_ORDER: for threads on service orders
 * - MPS_OFFER: for threads on services
 * Note: this resource supports seek pagination [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#seek-pagination}
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} [request.params]
 * @param {string} [request.params.entity_type]
 * @param {string} [request.params.entity_id]
 * @param {number} [request.params.limit]
 * @param {string} [request.params.previous_page_token]
 * @param {string} [request.params.next_page_token]
 */
const getListAllThreads = ({ url, headers, params }) => {
  const { error, value } = Joi.object()
    .keys({
      entity_type: Joi.string()
        .valid('MMP_ORDER', 'MMP_OFFER', 'MPS_ORDER', 'MPS_SERVICE')
        .optional(),
      entity_id: Joi.array().items(Joi.string()).optional(),
      ...seekPaginationSchema,
    })
    .optional()
    .validate(params);
  if (error) {
    return validationError(error.details);
  }
  if (value) {
    if (value.entity_id && value.entity_id.length) {
      value.entity_id = value.entity_id.join(',');
    } else {
      delete value.entity_id;
    }
  }

  return client.getRequest({
    url: url + MESSAGES_PATH,
    headers,
    params: value,
  });
};

/**
 * Create a thread on an order [OR43] {@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR43}
 * Each message has a 10 Megabyte size limit, regardless of the number of attachments in the message.
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {string} request.id
 */
const createThreadByOrderId = ({
  url, headers, orderId, data
}) => {
  const validatedID = Joi.string().required().validate(orderId);
  if (validatedID.error) {
    return validationError(validatedID.error.details);
  }
  const { error, value } = createThreadSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  const bodyFormData = new FormData();
  if (value.files) {
    data.files.forEach((f) => bodyFormData.append('files', f));
  }
  try {
    bodyFormData.append('thread_input', JSON.stringify(value.thread_input), {
      contentType: 'application/json',
    });
  } catch (e) {
    return failure(e.message, e);
  }

  return client.postRequest({
    url: url + MESSAGES_ORDERS_PATH(validatedID.value),
    headers: {
      ...headers,
      'Content-Type': 'multipart/form-data',
      ...bodyFormData.getHeaders(),
    },
    data: bodyFormData,
  });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient' || name === 'FrontClient') {
      ctx.prototype.getListAllThreads = function (params) {
        return getListAllThreads({ ...this._config, params });
      };
      ctx.prototype.createThreadByOrderId = function ({ orderId, data }) {
        return createThreadByOrderId({ ...this._config, orderId, data });
      };
    }
  }
};
