const FormData = require('form-data');
const Joi = require('joi');
const client = require('../core/client');
const { validationError } = require('../core/result');

const OFFERS_PATH = '/api/offers';
const EXPORT_OFFERS = `${OFFERS_PATH}/export`;
const OFFER_INFO = (id) => `${OFFERS_PATH}/${id}`;
const OFFERS_STATES_PATH = `${OFFERS_PATH}/states`;
const OFFERS_IMPORT_PATH = `${OFFERS_PATH}/imports`;
const OFFER_IMPORT_STATUS_PATH = (id) => `${OFFERS_IMPORT_PATH}/${id}`;
const OFFER_IMPORT_ERROR_REPORT_PATH = (id) => `${OFFER_IMPORT_STATUS_PATH(id)}/error_report`;

const importOffersSchema = Joi.object().keys({
  file: Joi.object({ pipe: Joi.func().required() }).unknown().required(),
  import_mode: Joi.string()
    .valid('NORMAL', 'PARTIAL_UPDATE', 'REPLACE')
    .required(),
  shop: Joi.number().positive().required(),
  with_products: Joi.boolean().optional(),
});

/**
 * Import a file to add offers [POST OF01]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OF01}
 * Returns the import identifier to track the status of the import
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.data
 * @param {number} request.data.shop - Shop ID
 * @param {Object} request.data.file - offer info in file stream
 * @param {string} request.data.import_mode - One of NORMAL, PARTIAL_UPDATE, REPLACE
 * @param {string} request.data.with_products - true if upload offers and products at the same time
 * @returns {Promise} Promise object
 */
const importOffers = async ({ url, headers, data }) => {
  const { error, value } = importOffersSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  const bodyFormData = new FormData();
  bodyFormData.append('shop', value.shop);
  bodyFormData.append('file', data.file);
  bodyFormData.append('import_mode', value.import_mode);
  if (value.with_products) {
    bodyFormData.append('with_products', value.with_products);
  }
  return client.postRequest({
    url: url + OFFERS_IMPORT_PATH,
    headers: {
      ...headers,
      'Content-Type': 'multipart/form-data',
      ...bodyFormData.getHeaders(),
    },
    data: bodyFormData,
  });
};

/**
 * Get information and statistics about an offer import [GET OF02]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OF02}
 * Recommended: After each OF01 call, every 5 minutes
 * Maximum: Once per minute
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {number} request.importId
 * @returns {Promise} Promise object
 */
const getStatusForOffersImportByImportId = async ({
  url,
  headers,
  importId,
}) => {
  const { error, value } = Joi.number().positive().validate(importId);
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + OFFER_IMPORT_STATUS_PATH(value),
    headers,
  });
};

/**
 * Get the error report file for an offer import [GET OF03]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OF03}
 * Recommended: After each OF02 call, every 5 minutes
 * Maximum: Once per minute
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {number} request.importId
 * @returns {Promise<{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean}|{data: *, isOk: boolean}|undefined>} Promise object
 */
const getErrorReportForOfferImportByImportId = ({ url, headers, importId }) => {
  const { error, value } = Joi.number().positive().validate(importId);
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + OFFER_IMPORT_ERROR_REPORT_PATH(value),
    headers: { ...headers, Accept: 'application/csv' },
  });
};

/**
 * List offer conditions [GET OF61]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OF61}
 * Sorted by sort index, set in the back-office
 * Note: this resource supports locale parameter [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#locale})
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {boolean} [request.active]
 * @param {string} [request.locale]
 * @returns {Promise<{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean}|{data: *, isOk: boolean}|undefined>} Promise object
 */
const getListOfferConditions = ({
  url, headers, active, locale
}) => {
  const { error, value } = Joi.object()
    .keys({
      active: Joi.boolean().optional(),
      locale: Joi.string().optional(),
    })
    .validate({ active, locale });
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + OFFERS_STATES_PATH,
    headers,
    params: value,
  });
};

/**
 * Get information about an offer [GET OF22]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OF22}
 * Note: this resource supports locale parameter [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#locale})
 * Recommended: At each offer page display
 * Maximum: At each offer page display
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {number} request.offerId
 * @param {string} [request.pricing_channel_code]
 * @param {string} [request.locale]
 * @returns {Promise} Promise object
 */
const getOfferByOfferId = ({
  url,
  headers,
  offerId,
  pricing_channel_code,
  locale,
}) => {
  const { error, value } = Joi.object()
    .keys({
      offerId: Joi.number().positive().required(),
      pricing_channel_code: Joi.string().optional().allow(null, ''),
      locale: Joi.string().optional(),
    })
    .validate({ offerId, pricing_channel_code, locale });
  if (error) {
    return validationError(error.details);
  }
  const params = { ...value };
  delete params.offerId;
  return client.getRequest({
    url: url + OFFER_INFO(value.offerId),
    headers,
    params,
  });
};

/**
 * Get offers CSV file [GET OF51]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OF51}
 * Get a CSV file that includes the offers updated and deleted since the last request date
 * If the last_request_date param is not set the api returns all the active offers (inactive offers can be included with include_inactive_offers parameter).
 * If offers have custom fields, a column is added at the end of csv for each existing custom field with as header the code of the custom field.
 * Recommended:
 * - Differential: every 5 minutes
 * - Full: once per day
 * Maximum:
 * - Differential: once per minute
 * - Full: once per day
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {string} [request.last_request_date] - date-time format
 * @param {string} [request.channel_codes]
 * @param {boolean} [request.include_inactive_offers]
 */
const exportOffers = ({
  url,
  headers,
  last_request_date,
  channel_codes,
  include_inactive_offers,
}) => {
  const { error, value } = Joi.object()
    .keys({
      last_request_date: Joi.date().iso().optional(),
      channel_codes: Joi.string().optional(),
      include_inactive_offers: Joi.boolean().optional(),
    })
    .optional()
    .validate({ last_request_date, channel_codes, include_inactive_offers });
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + EXPORT_OFFERS,
    headers,
    params: value,
  });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient') {
      ctx.prototype.importOffers = function (data) {
        return importOffers({ ...this._config, data });
      };
      ctx.prototype.getStatusForOffersImportByImportId = function (
        importId
      ) {
        return getStatusForOffersImportByImportId({
          ...this._config,
          importId,
        });
      };
      ctx.prototype.getErrorReportForOfferImportByImportId = function (
        importId
      ) {
        return getErrorReportForOfferImportByImportId({
          ...this._config,
          importId,
        });
      };
      ctx.prototype.getListOfferConditions = function (params) {
        return getListOfferConditions({ ...this._config, ...params });
      };
      ctx.prototype.getOfferByOfferId = function (params) {
        return getOfferByOfferId({ ...this._config, ...params });
      };
      ctx.prototype.exportOffers = function (params) {
        return exportOffers({ ...this._config, ...params });
      };
    }
  }
};
