const FormData = require('form-data');
const Joi = require('joi').extend(require('joi-currency-code'));
const client = require('../core/client');
const { validationError, failure } = require('../core/result');
const {
  getCountryISOCodes,
  getLocaleShortCodes,
  getCurrencyISOCodes,
} = require('../core/utils');

const ORDERS_PATH = '/api/orders';
const REFUND_ORDERS_PATH = `${ORDERS_PATH}/refund`;
const ORDERS_DOCUMENTS_PATH = `${ORDERS_PATH}/documents`;
const DOWNLOAD_ORDERS_DOCUMENTS_PATH = `${ORDERS_DOCUMENTS_PATH}/download`;
const ORDERS_INFO_PATH = (id) => `${ORDERS_PATH}/${id}`;
const ORDERS_LINE_PATH = (id, line) => `${ORDERS_INFO_PATH(id)}/lines/${line}`;
const OPEN_INCIDENT_ORDERS_PATH = (id, line) => `${ORDERS_LINE_PATH(id, line)}/open_incident`;
const CLOSE_INCIDENT_ORDERS_PATH = (id, line) => `${ORDERS_LINE_PATH(id, line)}/close_incident`;
const EVALUATION_ORDERS_PATH = (id) => `${ORDERS_INFO_PATH(id)}/evaluation`;
const VALIDATE_RECEIPT_ORDERS_PATH = (id) => `${ORDERS_INFO_PATH(id)}/receive`;
const UPLOAD_DOCS_ORDERS_PATH = (id) => `${ORDERS_INFO_PATH(id)}/documents`;
const VALIDATE_SHIPMENT_ORDERS_PATH = (id) => `${ORDERS_INFO_PATH(id)}/ship`;
const TRACKING_ORDERS_PATH = (id) => `${ORDERS_INFO_PATH(id)}/tracking`;
const ACCEPT_REFUSE_ORDERS_PATH = (id) => `${ORDERS_INFO_PATH(id)}/accept`;
const VALIDATE_COMMERCIAL_ORDER = (id) => `${ORDERS_PATH}/valid/${id}`;
const INVALIDATE_COMMERCIAL_ORDER = (id) => `${ORDERS_PATH}/invalid/${id}`;

const localeSchema = Joi.string()
  .valid(...getLocaleShortCodes)
  .optional();

const paginationSchema = {
  max: Joi.number().positive().max(100).optional()
    .default(10),
  offset: Joi.number().min(0).optional().default(0),
  sort: Joi.string().optional(),
  order: Joi.string().valid('asc', 'desc').optional().default('asc'),
};

const refundOrderLineSchema = Joi.object()
  .keys({
    order_tax_mode: Joi.string()
      .valid('TAX_EXCLUDED', 'TAX_INCLUDED')
      .optional(),
    refunds: Joi.array()
      .items(
        Joi.object()
          .keys({
            amount: Joi.number().required(),
            amount_breakdown: Joi.object()
              .keys({
                parts: Joi.array()
                  .items({
                    amount: Joi.number().min(0).required(),
                    commissionable: Joi.boolean().optional(),
                    debitable_from_customer: Joi.boolean().optional(),
                    payable_to_shop: Joi.boolean().optional(),
                  })
                  .required(),
              })
              .optional(),
            currency_iso_code: Joi.string()
              .valid(...getCurrencyISOCodes)
              .optional(),
            order_line_id: Joi.string().required(),
            quantity: Joi.number().optional(),
            reason_code: Joi.string().required(),
            shipping_amount: Joi.number().required(),
            shipping_amount_breakdown: Joi.object()
              .keys({
                parts: Joi.array()
                  .items({
                    amount: Joi.number().min(0).required(),
                    commissionable: Joi.boolean().optional(),
                    debitable_from_customer: Joi.boolean().optional(),
                    payable_to_shop: Joi.boolean().optional(),
                  })
                  .required(),
              })
              .optional(),
            shipping_taxes: Joi.array()
              .items({
                amount: Joi.number().required(),
                amount_breakdown: Joi.object()
                  .keys({
                    parts: Joi.array()
                      .items({
                        amount: Joi.number().min(0).required(),
                        commissionable: Joi.boolean().optional(),
                        debitable_from_customer: Joi.boolean().optional(),
                        payable_to_shop: Joi.boolean().optional(),
                      })
                      .required(),
                  })
                  .optional(),
                code: Joi.string().required(),
              })
              .optional(),
            taxes: Joi.array()
              .items({
                amount: Joi.number().required(),
                amount_breakdown: Joi.object()
                  .keys({
                    parts: Joi.array()
                      .items({
                        amount: Joi.number().min(0).required(),
                        commissionable: Joi.boolean().optional(),
                        debitable_from_customer: Joi.boolean().optional(),
                        payable_to_shop: Joi.boolean().optional(),
                      })
                      .required(),
                  })
                  .optional(),
                code: Joi.string().required(),
              })
              .optional(),
          })
          .required()
      )
      .required(),
  })
  .required();

const listOrdersQuerySchema = Joi.object().keys({
  commercial_ids: Joi.array().items(Joi.string()).optional(),
  customer_ids: Joi.array().items(Joi.string()).optional(),
  offer_ids: Joi.array().items(Joi.string()).optional(),
  order_ids: Joi.array().items(Joi.string()).optional(),
  shop_ids: Joi.array().items(Joi.string()).optional(),
  product_skus: Joi.array().items(Joi.string()).optional(),
  order_state_codes: Joi.array().items(Joi.string()).optional(),
  channel_codes: Joi.array().items(Joi.string()).optional(),
  only_null_channel: Joi.boolean().default(false),
  start_date: Joi.date().iso().optional(),
  end_date: Joi.date().iso().optional(),
  start_update_date: Joi.date().iso().optional(),
  end_update_date: Joi.date().iso().optional(),
  paginate: Joi.boolean().default(true),
  customer_debited: Joi.boolean().optional(),
  payment_workflow: Joi.string()
    .valid('PAY_ON_ACCEPTANCE', 'PAY_ON_DELIVERY', 'PAY_ON_DUE_DATE')
    .optional(),
  has_incident: Joi.boolean().optional(),
  fulfillment_center_code: Joi.array().items(Joi.string()).optional(),
  order_tax_mode: Joi.string().valid('TAX_EXCLUDED', 'TAX_INCLUDED').optional(),
  locale: localeSchema,
  ...paginationSchema,
});

const createOrdersSchema = Joi.object().keys({
  channel_code: Joi.string().required(),
  commercial_id: Joi.string().required(),
  customer: Joi.object().keys({
    billing_address: Joi.object()
      .keys({
        city: Joi.string().required(),
        civility: Joi.string().allow(null, '').optional(),
        company: Joi.string().allow(null, '').optional(),
        country: Joi.string().required(),
        country_iso_code: Joi.string()
          .valid(...getCountryISOCodes)
          .required(),
        firstname: Joi.string().optional(),
        lastname: Joi.string().required(),
        phone: Joi.string().allow(null, '').optional(),
        phone_secondary: Joi.string().allow(null, '').optional(),
        state: Joi.string().allow(null, '').optional(),
        street_1: Joi.string().required(),
        street_2: Joi.string().allow(null, '').optional(),
        zip_code: Joi.string().optional(),
      })
      .required(),
    civility: Joi.string().required(),
    customer_id: Joi.string().required(),
    email: Joi.string().allow(null, '').optional(),
    firstname: Joi.string().required(),
    lastname: Joi.string().required(),
    locale: Joi.string()
      .valid(...getLocaleShortCodes)
      .required(),
    shipping_address: Joi.object()
      .keys({
        additional_info: Joi.string().allow(null, '').optional(),
        city: Joi.string().required(),
        civility: Joi.string().allow(null, '').optional(),
        company: Joi.string().allow(null, '').optional(),
        country: Joi.string().required(),
        country_iso_code: Joi.string()
          .valid(...getCountryISOCodes)
          .required(),
        firstname: Joi.string().optional(),
        internal_additional_info: Joi.string().allow(null, '').optional(),
        lastname: Joi.string().required(),
        phone: Joi.string().allow(null, '').optional(),
        phone_secondary: Joi.string().allow(null, '').optional(),
        state: Joi.string().allow(null, '').optional(),
        street_1: Joi.string().required(),
        street_2: Joi.string().allow(null, '').optional(),
        zip_code: Joi.string().optional(),
      })
      .required(),
  }),
  offers: Joi.array().items({
    adjustment_limit: Joi.number().optional(),
    currency_iso_code: Joi.string()
      .valid(...getCurrencyISOCodes)
      .optional(),
    delivery_date: Joi.object()
      .keys({
        earliest: Joi.date().iso(),
        latest: Joi.date().iso(),
      })
      .optional(),
    leadtime_to_ship: Joi.number().optional(),
    offer_id: Joi.number().required(),
    offer_price: Joi.number().required(),
    order_line_additional_fields: Joi.array()
      .items({
        code: Joi.string().required(),
        value: Joi.string().required(),
      })
      .optional()
      .default([]),
    order_line_id: Joi.string().optional(),
    price: Joi.number().required(),
    quantity: Joi.number().positive().required(),
    shipping_deadline: Joi.date().iso().allow(null).optional(),
    shipping_from: Joi.object().keys({
      address: Joi.object().keys({
        city: Joi.string().required(),
        country_iso_code: Joi.string()
          .valid(...getCountryISOCodes)
          .required(),
        state: Joi.string().allow(null, '').optional(),
        street_1: Joi.string().allow(null, '').optional(),
        street_2: Joi.string().allow(null, '').optional(),
        zip_code: Joi.string().optional(),
      }).optional()
    }).optional(),
    shipping_price: Joi.number().required(),
    shipping_pudo_id: Joi.string().allow(null).optional(),
    shipping_taxes: Joi.array()
      .items({
        amount: Joi.number().required(),
        code: Joi.string().required(),
        rate: Joi.number().optional(),
      })
      .optional()
      .default([]),
    shipping_type_code: Joi.string().required(),
    taxes: Joi.array()
      .items({
        amount: Joi.number().required(),
        code: Joi.string().required(),
        rate: Joi.number().optional(),
      })
      .optional()
      .default([]),
  }),
  order_additional_fields: Joi.array()
    .items({
      code: Joi.string().required(),
      value: Joi.string().required(),
    })
    .optional()
    .default([]),
  order_tax_mode: Joi.string().valid('TAX_INCLUDED', 'TAX_EXCLUDED').optional(),
  payment_duration: Joi.number().optional(),
  payment_info: Joi.object()
    .keys({
      imprint_number: Joi.string().optional(),
      payment_type: Joi.string().optional(),
    })
    .required(),
  payment_workflow: Joi.string().valid('PAY_ON_ACCEPTANCE', 'PAY_ON_DELIVERY', 'PAY_ON_DUE_DATE', 'PAY_ON_SHIPMENT', 'NO_CUSTOMER_PAYMENT_CONFIRMATION').optional(),
  references: Joi.object().keys({
    order_reference_for_customer: Joi.string().optional()
  }).optional(),
  scored: Joi.boolean().required(),
  shipping_zone_code: Joi.string().required(),
});

const jointByComma = (obj, name) => {
  if (obj[name] && obj[name].length) {
    obj[name] = obj[name].join(',');
  }
};

const uploadDocumentSchema = Joi.object().keys({
  files: Joi.array()
    .items(Joi.object({ pipe: Joi.func().required() }).unknown().required())
    .max(50)
    .required(),
  order_documents: Joi.array()
    .items(
      Joi.object()
        .keys({
          file_name: Joi.string().required(),
          type_code: Joi.string().required(),
        })
        .required()
    )
    .required(),
});

const evaluationSchema = Joi.object()
  .keys({
    assessments: Joi.array()
      .items(
        Joi.object().keys({
          code: Joi.string().required(),
          response: Joi.string().required(),
        })
      )
      .required(),
    comment: Joi.string().optional(),
    grade: Joi.number().min(0).max(5).required(),
    visible: Joi.boolean().optional(),
  })
  .required();

const incidentSchema = Joi.object()
  .keys({
    id: Joi.string().required(),
    line: Joi.string().required(),
    data: Joi.object()
      .keys({
        reason_code: Joi.string().required(),
      })
      .required(),
  })
  .required();

/**
 * Create an order [POST OR01]{@link https://help.mirakl.net/help/api-doc/front/mmp.html#OR01}
 * The leadtime_to_ship field is not mandatory. Fill it only to override the original offer lead time to ship and the configured default lead time to ship.
 * If not specified in the API call, Mirakl will apply the offer's leadtime to ship.
 * If the offer has a lead time to ship of value 0, Mirakl will apply a lead time to ship of 1.
 * And if the offer does not have a lead time to ship, Mirakl will apply the configured default lead time to ship.
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.data {@link https://help.mirakl.net/help/api-doc/front/mmp.html#OR01}
 */
const createOrders = ({ url, headers, data }) => {
  const { error, value } = createOrdersSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client
    .postRequest({
      url: url + ORDERS_PATH,
      headers,
      data: value,
    })
    .then((res) => {
      if (res.isOk && res.data.offers_not_shippable.length) {
        return failure(res.data.offers_not_shippable[0].error_code, res.data);
      }
      return res;
    });
};

/**
 * List orders [GET OR11]{@link https://help.mirakl.net/help/api-doc/front/mmp.html#OR11}
 * Returns 400 Bad Request if the limit of commercialIds or customerIds or
 * offerIds or productSkus is exceeded.
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.params {@link https://help.mirakl.net/help/api-doc/front/mmp.html#OR11}
 */
const getListOrders = ({ url, headers, params }) => {
  const { error, value } = listOrdersQuerySchema.validate(params);
  if (error) {
    return validationError(error.details);
  }
  [
    'commercial_ids',
    'customer_ids',
    'offer_ids',
    'order_ids',
    'shop_ids',
    'product_skus',
    'order_state_codes',
    'channel_codes',
  ].forEach((f) => jointByComma(value, f));

  return client.getRequest({
    url: url + ORDERS_PATH,
    headers,
    params: value,
  });
};

/**
 * Validate a commercial order [PUT OR02]{@link https://help.mirakl.net/help/api-doc/front/mmp.html#OR02}
 * Validate a commercial order in the STAGING status
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {string} request.commercial_id
 */
const validateCommercialOrder = ({ url, headers, commercial_id }) => {
  const { error, value } = Joi.string().required().validate(commercial_id);
  if (error) {
    return validationError(error.details);
  }
  return client.putRequest({
    url: url + VALIDATE_COMMERCIAL_ORDER(value),
    headers,
  });
};

/**
 * Invalidate a commercial order [PUT OR03]{@link https://help.mirakl.net/help/api-doc/front/mmp.html#OR03}
 * Invalidate a commercial order in the STAGING status
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {string} request.commercial_id
 */
const invalidateCommercialOrder = ({ url, headers, commercial_id }) => {
  const { error, value } = Joi.string().required().validate(commercial_id);
  if (error) {
    return validationError(error.details);
  }
  return client.putRequest({
    url: url + INVALIDATE_COMMERCIAL_ORDER(value),
    headers,
  });
};

/**
 * Accept or refuse order lines [PUT OR21]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR21}
 * Accept or refuse order lines in the WAITING_ACCEPTANCE status
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {string}   request.id - Order's identifier
 * @param {Object}   request.data
 * @param {Object[]} request.data.order_lines
 * @param {boolean}  request.data.order_lines[].accepted
 * @param {string}   request.data.order_lines[].id - Order line's identifier
 */
const acceptOrRefuseOrderLines = ({
  url, headers, id, data
}) => {
  const validatedID = Joi.string().required().validate(id);
  if (validatedID.error) {
    return validationError(validatedID.error.details);
  }
  const { error, value } = Joi.object()
    .keys({
      order_lines: Joi.array()
        .items({
          accepted: Joi.boolean().required(),
          id: Joi.string().required(),
        })
        .required(),
    })
    .required()
    .validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client.putRequest({
    url: url + ACCEPT_REFUSE_ORDERS_PATH(validatedID.value),
    headers,
    data: value,
  });
};

/**
 * Update carrier tracking information for a specific order [PUT OR23] {@link https://help.mirakl.net/help/api-doc/front/mmp.html#OR23}
 * If the carrier is not registered, the complete tracking url can be provided.
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {string} request.id - Order's identifier
 * @param {Object} request.data
 * @param {string }
 */
const updateTrackingInfoByOrderId = ({
  url, headers, id, data
}) => {
  const validatedID = Joi.string().required().validate(id);
  if (validatedID.error) {
    return validationError(validatedID.error.details);
  }
  const { error, value } = Joi.object()
    .keys({
      carrier_code: Joi.string().optional(),
      carrier_name: Joi.string().optional(),
      carrier_url: Joi.string().optional(),
      tracking_number: Joi.string().optional(),
    })
    .optional()
    .default({})
    .validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client.putRequest({
    url: url + TRACKING_ORDERS_PATH(validatedID.value),
    headers,
    data: value,
  });
};

/**
 * Validate the shipment of an order [PUT OR24] {@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR24}
 * Validate the shipment of an order in the SHIPPING status
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {string} request.id - Order's identifier
 */
const validateShipmentByOrderId = ({ url, headers, id }) => {
  const validatedID = Joi.string().required().validate(id);
  if (validatedID.error) {
    return validationError(validatedID.error.details);
  }
  return client.putRequest({
    url: url + VALIDATE_SHIPMENT_ORDERS_PATH(validatedID.value),
    headers,
  });
};

/**
 * Upload documents to attach to an order [POST OR74]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR74}
 * Documents filenames must be distinct. Only documents of the
 * following types are supported: csv, doc, xls, xlsx, ppt, pdf, odt, ods,
 * odp, txt, rtf, png, jpg, gif, zpl, mov, mp4.
 * An order can have a maximum of 50 documents. In the API output
 * we include only documents with errors. All other documents are successfully uploaded.
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {string}   request.id - Order's identifier
 * @param {Object}   request.data
 * @param {Object[]} request.data.files
 * @param {Object[]} request.data.order_documents
 * @param {string}   request.data.order_documents[].file_name
 * @param {string}   request.data.order_documents[].type_code
 */
const uploadDocumentsByOrderId = ({
  url, headers, id, data
}) => {
  const validatedID = Joi.string().required().validate(id);
  if (validatedID.error) {
    return validationError(validatedID.error.details);
  }
  const { error, value } = uploadDocumentSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  const bodyFormData = new FormData();
  data.files.forEach((f) => bodyFormData.append('files', f));
  try {
    bodyFormData.append(
      'order_documents',
      JSON.stringify({ order_documents: value.order_documents }),
      { contentType: 'application/json' }
    );
  } catch (e) {
    return failure(e.message, e);
  }
  return client
    .postRequest({
      url: url + UPLOAD_DOCS_ORDERS_PATH(validatedID.value),
      headers: {
        ...headers,
        'Content-Type': 'multipart/form-data',
        ...bodyFormData.getHeaders(),
      },
      data: bodyFormData,
    })
    .then((res) => {
      if (res.isOk && res.data.errors_count) {
        return failure(
          res.data.order_documents
            .flatMap((obj) => obj.errors.flat())
            .map((e) => e.message)[0],
          res.data
        );
      }
      return res;
    });
};

/**
 * Validate the receipt of an order [OR25]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR25}
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {string}   request.id - Order's identifier
 */
const validateReceiptByOrderId = ({ url, headers, id }) => {
  const validatedID = Joi.string().required().validate(id);
  if (validatedID.error) {
    return validationError(validatedID.error.details);
  }
  return client.putRequest({
    url: url + VALIDATE_RECEIPT_ORDERS_PATH(id),
    headers,
  });
};

/**
 * Get the evaluation of an order [OR51] {@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR51}
 * Note: this resource supports locale parameter [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#locale}
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {string}   request.id - Order's identifier
 * @param {string}   request.locale
 */
const getEvaluationByOrderId = ({
  url, headers, id, locale
}) => {
  const { error, value } = Joi.object()
    .keys({
      id: Joi.string().required(),
      locale: localeSchema,
    })
    .required()
    .validate({ id, locale });
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + EVALUATION_ORDERS_PATH(value.id),
    headers,
    params: value.locale ? { locale: value.locale } : {},
  });
};

/**
 * Post an evaluation on an order [OR52]{@link https://help.mirakl.net/help/api-doc/front/mmp.html#OR52}
 * Recommended: At each order evaluation
 * Maximum: At each order evaluation
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {string}   request.id - Order's identifier
 * @param {Object}   request.data
 * @param {Object[]} request.data.assessments
 * @param {string}   request.data.assessments[].code
 * @param {string}   request.data.assessments[].response
 * @param {string}   [request.data.comment]
 * @param {number}   request.data.grade
 * @param {boolean}  [request.data.visible]
 */
const evaluateByOrderId = ({
  url, headers, id, data
}) => {
  const validatedID = Joi.string().required().validate(id);
  if (validatedID.error) {
    return validationError(validatedID.error.details);
  }
  const { error, value } = evaluationSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client.postRequest({
    url: url + EVALUATION_ORDERS_PATH(validatedID.value),
    headers,
    data: value,
  });
};

/**
 * Post an incident on an order line [OR62]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR62}
 * Recommended: At each incident request on an order line
 * Maximum: At each incident request on an order line
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {string}   request.id - Order's identifier
 * @param {string}   request.line - Order line
 * @param {Object}   request.data
 * @param {string}   request.data.reason_code
 */
const openIncidentByOrderLine = ({
  url, headers, id, line, data
}) => {
  const { error, value } = incidentSchema.validate({ id, line, data });
  if (error) {
    return validationError(error.details);
  }
  return client.postRequest({
    url: url + OPEN_INCIDENT_ORDERS_PATH(value.id, value.line),
    headers,
    data: value.data,
  });
};

/**
 * Close an incident on an order line [OR63]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR63}
 * Recommended: At each incident closure on an order line
 * Maximum: At each incident closure on an order line
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {string}   request.id - Order's identifier
 * @param {string}   request.line - Order line
 * @param {Object}   request.data
 * @param {string}   request.data.reason_code
 */
const closeIncidentByOrderLine = ({
  url, headers, id, line, data
}) => {
  const { error, value } = incidentSchema.validate({ id, line, data });
  if (error) {
    return validationError(error.details);
  }
  return client.putRequest({
    url: url + CLOSE_INCIDENT_ORDERS_PATH(value.id, value.line),
    headers,
    data: value.data,
  });
};

/**
 * Lists order's documents [OR72]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR72}
 * Returns a list of all the documents available on the order
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {string[]}   request.ids - Order's identifiers
 */
const getListOrderDocumentsByOrderIds = ({ url, headers, ids }) => {
  const { error, value } = Joi.array()
    .items(Joi.string().required())
    .required()
    .validate(ids);
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + ORDERS_DOCUMENTS_PATH,
    headers,
    params: { order_ids: value.join(',') },
  });
};

/**
 * Download one or multiple documents attached to one or multiple orders [OR73]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR73}
 *  - If a list of document identifiers is specified only these documents are downloaded.
 *      - If more than one document id is specified, the documents will be wrapped in a ZIP archive
 *      - If only one document id is specified the document will not be zipped
 *  - If a list of order identifiers is specified, all documents from those orders are downloaded.
 *    Use a list of order document type codes to retrieve specific types from those orders.
 *    In this case, the output of the API will always be a ZIP archive even if there is only one document to retrieve.
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {Object}   request.params
 * @param {string[]} request.params.order_ids
 * @param {string[]} request.params.document_ids
 * @param {string[]} request.params.document_codes
 */
const downloadDocumentsByOrderIds = ({ url, headers, params }) => {
  const { error, value } = Joi.object({
    order_ids: Joi.array().items(Joi.string().required()).optional(),
    document_ids: Joi.array().items(Joi.string().required()).optional(),
    document_codes: Joi.array().items(Joi.string().required()).optional(),
  })
    .min(1)
    .validate(params);
  if (error) {
    return validationError(error.details);
  }
  const newParams = {};
  if (value.order_ids && value.order_ids.length) {
    newParams.order_ids = value.order_ids.join(',');
  }
  if (value.document_ids && value.document_ids.length) {
    newParams.document_ids = value.document_ids.join(',');
  }
  if (value.document_codes && value.document_codes.length) {
    newParams.document_codes = value.document_codes.join(',');
  }
  return client.getRequest({
    url: url + DOWNLOAD_ORDERS_DOCUMENTS_PATH,
    headers,
    params: newParams,
  });
};

/**
 * Request refunds on order lines [OR28]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#OR28}
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {Object}   request.data
 */
const refundOrderLine = ({ url, headers, data }) => {
  const { error, value } = refundOrderLineSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client.putRequest({
    url: url + REFUND_ORDERS_PATH,
    headers,
    data: value,
  });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient' || name === 'FrontClient') {
      ctx.prototype.downloadDocumentsByOrderIds = function ({
        orderIds,
        documentIds,
        documentCodes,
      }) {
        return downloadDocumentsByOrderIds({
          ...this._config,
          params: {
            order_ids: orderIds,
            document_ids: documentIds,
            document_codes: documentCodes,
          },
        });
      };
      ctx.prototype.getListOrderDocumentsByOrderIds = function (orderIds) {
        return getListOrderDocumentsByOrderIds({
          ...this._config,
          ids: orderIds,
        });
      };
      ctx.prototype.closeIncidentByOrderLine = function ({
        orderId,
        line,
        data,
      }) {
        return closeIncidentByOrderLine({
          ...this._config,
          id: orderId,
          line,
          data,
        });
      };
      ctx.prototype.openIncidentByOrderLine = function ({
        orderId,
        line,
        data,
      }) {
        return openIncidentByOrderLine({
          ...this._config,
          id: orderId,
          line,
          data,
        });
      };
      ctx.prototype.getEvaluationByOrderId = function ({ orderId, locale }) {
        return getEvaluationByOrderId({ ...this._config, id: orderId, locale });
      };
      ctx.prototype.validateReceiptByOrderId = function (orderId) {
        return validateReceiptByOrderId({ ...this._config, id: orderId });
      };
      ctx.prototype.uploadDocumentsByOrderId = function ({ orderId, data }) {
        return uploadDocumentsByOrderId({ ...this._config, id: orderId, data });
      };
      ctx.prototype.acceptOrRefuseOrderLines = function ({ orderId, data }) {
        return acceptOrRefuseOrderLines({ ...this._config, id: orderId, data });
      };
      ctx.prototype.getListOrders = function (params) {
        return getListOrders({ ...this._config, params });
      };
      ctx.prototype.updateTrackingInfoByOrderId = function ({
        orderId,
        data,
      }) {
        return updateTrackingInfoByOrderId({
          ...this._config,
          id: orderId,
          data,
        });
      };
      ctx.prototype.validateShipmentByOrderId = function (orderId) {
        return validateShipmentByOrderId({ ...this._config, id: orderId });
      };
    }
    if (name === 'FrontClient') {
      ctx.prototype.createOrders = function (data) {
        return createOrders({ ...this._config, data });
      };

      ctx.prototype.validateCommercialOrder = function (commercialId) {
        return validateCommercialOrder({
          ...this._config,
          commercial_id: commercialId,
        });
      };
      ctx.prototype.invalidateCommercialOrder = function (commercialId) {
        return invalidateCommercialOrder({
          ...this._config,
          commercial_id: commercialId,
        });
      };
      ctx.prototype.evaluateByOrderId = function ({ orderId, data }) {
        return evaluateByOrderId({ ...this._config, id: orderId, data });
      };
    }
    if (name === 'OperatorClient') {
      ctx.prototype.refundOrderLine = function (data) {
        return refundOrderLine({ ...this._config, data });
      };
    }
  }
};
