const Joi = require('joi').extend(require('joi-currency-code'));
const client = require('../core/client');
const { validationError } = require('../core/result');
const { getCurrencyISOCodes } = require('../core/utils');

const PAYMENT_PATH = '/api/payment';
const REFUND_PAYMENT_PATH = `${PAYMENT_PATH}/refund`;
const DEBIT_PAYMENT_PATH = `${PAYMENT_PATH}/debit`;

const debitPaymentSchema = Joi.object()
  .keys({
    orders: Joi.array()
      .items(
        Joi.object()
          .keys({
            amount: Joi.number().optional(),
            currency_iso_code: Joi.string().currency().optional(),
            customer_id: Joi.string().required(),
            order_id: Joi.string().required(),
            payment_status: Joi.string().valid('OK', 'REFUSED').required(),
            transaction_date: Joi.date().iso().optional(),
            transaction_number: Joi.string().optional(),
          })
          .required()
      )
      .required(),
  })
  .required();

const refundPaymentSchema = Joi.object()
  .keys({
    refunds: Joi.array()
      .items(
        Joi.object()
          .keys({
            amount: Joi.number().optional(),
            currency_iso_code: Joi.string()
              .valid(...getCurrencyISOCodes)
              .optional(),
            payment_status: Joi.string().valid('OK', 'REFUSED').required(),
            refund_id: Joi.string().required(),
            transaction_date: Joi.date().iso().optional(),
            transaction_number: Joi.string().optional(),
          })
          .required()
      )
      .required(),
  })
  .required();

/**
 * Validate or invalidate the payment of an order [PUT PA01]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#PA01}
 * If an error is returned (4xx, 5xx), everything is canceled.
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {Object}   request.data
 * @param {Object[]} request.data.orders
 * @param {number}   [request.data.orders[].amount]
 * @param {string}   [request.data.orders[].currency_iso_code]
 * @param {string}   request.data.orders[].customer_id
 * @param {string}   request.data.orders[].order_id
 * @param {string}   [request.data.orders[].payment_status]
 * @param {string}   [request.data.orders[].transaction_date]
 * @param {string}   [request.data.orders[].transaction_number]
 */
const validateOrInvalidatePaymentByOrder = ({ url, headers, data }) => {
  const { error, value } = debitPaymentSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client.putRequest({
    url: url + DEBIT_PAYMENT_PATH,
    headers,
    data: value,
  });
};

/**
 * Validate or invalidate the refund of order lines [PA02]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#PA02}
 * If an error is returned (4xx, 5xx), everything is canceled.
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {Object}   request.data
 * @param {Object[]} request.data.refunds
 * @param {number}   [request.data.refunds[].amount]
 * @param {string}   [request.data.refunds[].currency_iso_code]
 * @param {string}   request.data.refunds[].payment_status
 * @param {string}   request.data.refunds[].refund_id
 * @param {string}   [request.data.refunds[].transaction_date]
 * @param {string}   [request.data.refunds[].transaction_number]
 */
const refundPayment = ({ url, headers, data }) => {
  const { error, value } = refundPaymentSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client.putRequest({
    url: url + REFUND_PAYMENT_PATH,
    headers,
    data: value,
  });
};

/**
 * List pending order debits [PA11]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#PA11}
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {Object}   request.params
 */
const listPendingOrderDebits = ({ url, headers, params }) => {
  return client.getRequest({
    url: url + DEBIT_PAYMENT_PATH,
    params,
    headers,
  });
};

/**
 * List order refunds [PA12]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#PA12}
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {Object}   request.params
 */
const listPendingOrderRefunds = ({ url, headers, params }) => {
  return client.getRequest({
    url: url + REFUND_PAYMENT_PATH,
    params,
    headers,
  });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient' || name === 'FrontClient') {
      ctx.prototype.validateOrInvalidatePaymentByOrder = function (data) {
        return validateOrInvalidatePaymentByOrder({ ...this._config, data });
      };
      ctx.prototype.refundPayment = function (data) {
        return refundPayment({ ...this._config, data });
      };
      ctx.prototype.listPendingOrderDebits = function (params) {
        return listPendingOrderDebits({ ...this._config, params });
      };
      ctx.prototype.listPendingOrderRefunds = function (params) {
        return listPendingOrderRefunds({ ...this._config, params });
      };
    }
  }
};
