const Joi = require('joi');
const client = require('../core/client');
const { validationError } = require('../core/result');
const { getLocaleShortCodes } = require('../core/utils');

const CUSTOM_FIELDS_PATH = '/api/additional_fields';
const RATING_TYPES_PATH = '/api/evaluations/assessments';
const REASONS_PATH = '/api/reasons';

const localeSchema = Joi.string()
  .valid(...getLocaleShortCodes)
  .optional();

/**
 * List all custom fields [GET AF01]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#AF01}
 * Note: this resource supports locale parameter [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#locale}
 * Recommended: Once per day
 * Maximum: Once per day
 * @param {Object}   request
 * @param {string}   request.url
 * @param {Object}   request.headers
 * @param {Object}   [request.params]
 * @param {string[]} [request.params.entities]
 * @param {string}   [request.params.locale]
 */
const getListAllCustomFields = ({ url, headers, params }) => {
  const { error, value } = Joi.object()
    .keys({
      entities: Joi.array()
        .items(Joi.string())
        .has(
          Joi.string().valid('OFFER', 'SHOP', 'ORDER', 'ORDER_LINE').required()
        )
        .optional(),
      locale: localeSchema,
    })
    .optional()
    .validate(params);
  if (error) {
    return validationError(error.details);
  }
  if (value) {
    if (value.entities && value.entities.length) {
      value.entities = value.entities.join(',');
    } else {
      delete value.entities;
    }
  }

  return client.getRequest({
    url: url + CUSTOM_FIELDS_PATH,
    headers,
    params: value,
  });
};

/**
 * List rating types on your platform [EV01]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#EV01}
 * Assessments are sorted by sort index, set in the back-office
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} [request.params]
 * @param {string} [request.params.entity_type]
 * @param {string} [request.params.locale]
 */
const getListRatingTypes = ({ url, headers, params }) => {
  const { error, value } = Joi.object()
    .keys({
      entity_type: Joi.string().valid('MMP_ORDER', 'MPS_ORDER').optional(),
      locale: localeSchema,
    })
    .validate(params);
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + RATING_TYPES_PATH,
    headers,
    params: value,
  });
};

/**
 * List refund, incident, and message reasons [RE01]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#RE01}
 * Note: this resource supports locale parameter [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#locale}
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {string} [request.locale]
 */
const getListReasons = ({ url, headers, locale }) => {
  const { error, value } = localeSchema.validate(locale);
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + REASONS_PATH,
    headers,
    params: { ...value },
  });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient' || name === 'FrontClient') {
      ctx.prototype.getListReasons = function (locale) {
        return getListReasons({ ...this._config, locale });
      };
      ctx.prototype.getListRatingTypes = function (params) {
        return getListRatingTypes({ ...this._config, params });
      };
      ctx.prototype.getListAllCustomFields = function (entities) {
        return getListAllCustomFields({ ...this._config, entities });
      };
    }
  }
};
