/* eslint-disable max-len */
const FormData = require('form-data');
const Joi = require('joi').extend(require('joi-currency-code'));
const client = require('../core/client');
const { validationError } = require('../core/result');

const PRODUCT_PATH = '/api/products';
const PRODUCT_ATTRIBUTE_PATH = '/api/products/attributes';
const VALUE_LIST_PATH = '/api/values_lists';
const MCM_PRODUCT_PATH = '/api/mcm/products';
const MCM_PRODUCT_SYNC_PATH = `${MCM_PRODUCT_PATH}/synchronization`;
const PRODUCT_EXPORT_PATH = `${MCM_PRODUCT_PATH}/export`;
const PRODUCT_IMPORT_PATH = `${PRODUCT_PATH}/imports`;
const DELETED_PRODUCT_EXPORT_PATH = `${MCM_PRODUCT_PATH}/deleted/export`;
const PRODUCT_IMPORT_STATUS_PATH = (id) => `${PRODUCT_IMPORT_PATH}/${id}`;

const transformArrayToString = (arr) => {
  return arr && arr.length > 0 ? arr.join(',') : null;
};

const exportDeletedProductsQueryParamSchema = Joi.object().keys({
  deleted_from: Joi.date().iso().optional(),
  deleted_to: Joi.date().iso().optional(),
  product_id: Joi.array().items(Joi.string()).max(100).optional(),
}).allow(null)
  .optional();

const syncProductsSchema = Joi.array().items(Joi.object().keys({
  acceptance: Joi.object().keys({
    message: Joi.string().optional(),
    reason_code: Joi.string().optional(),
    status: Joi.string().required()
  }).optional(),
  catalogs: Joi.array().items(Joi.string()).optional(),
  data: Joi.object().optional(),
  integration_errors: Joi.array().items(Joi.object().keys(
    {
      attribute_code: Joi.string().optional(),
      code: Joi.string().optional(),
      message: Joi.string().optional()
    }
  )).optional(),
  mirakl_product_id: Joi.string().optional(),
  operation: Joi.string().valid('CREATE_UPDATE', 'DELETE').optional(),
  product_sku: Joi.string().optional(),
  product_urls: Joi.array().items(Joi.object().keys(
    {
      link: Joi.string().uri().required(),
      qualifier: Joi.string().required()
    }
  )).optional(),
  selling_authorization: Joi.object().keys({
    authorized_selling_shop_ids: Joi.array().items(Joi.string()).required(),
    restricted: Joi.boolean().required()
  }),
})).min(1).required();

const importProductSchema = Joi.object().keys({
  file: Joi.any().required(),
  shop: Joi.number().positive().required(),
});

const exportProductsQueryParamSchema = Joi.object()
  .keys({
    updated_since: Joi.date().iso().optional(),
    updated_to: Joi.date().iso().optional(),
    acceptance_status: Joi.array()
      .items(
        Joi.string()
          .valid(
            'NEW',
            'REJECTED',
            'ACCEPTED',
            'CHANGES_REQUESTED',
            'TO_REVIEW'
          )
          .default('ACCEPTED')
      )
      .max(100)
      .optional(),
    validation_status: Joi.array()
      .items(Joi.string().valid('VALID', 'INVALID').default('VALID'))
      .max(100)
      .optional(),
    synchronization_status: Joi.array()
      .items(
        Joi.string().valid('SYNCHRONIZED', 'PENDING', 'INTEGRATION_ERRORS')
      )
      .max(100)
      .optional(),
    product_sku: Joi.array().items(Joi.string()).max(100).optional(),
    product_id: Joi.array().items(Joi.string()).max(100).optional(),
    variant_group_code: Joi.array().items(Joi.string()).max(100).optional(),
    catalog: Joi.array().items(Joi.string()).max(100).optional(),
    rejection_reason: Joi.array().items(Joi.string()).max(100).optional(),
    category: Joi.array().items(Joi.string()).max(100).optional(),
  })
  .allow(null)
  .optional();

const exportAttributesSchema = Joi.object()
  .keys({
    hierarchy: Joi.string().optional(),
    max_level: Joi.number()
      .integer()
      .min(1).max(4)
      .optional(),
    with_roles: Joi.bool().optional(),
  })
  .allow(null)
  .optional();

const exportValueListSchema = Joi.object()
  .keys({
    code: Joi.string().optional(),
  })
  .allow(null)
  .optional();
/**
 * Import products to the operator information system [POST P41]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#P41}
 * Returns the import identifier to track the status of the import
 * Recommended: Every hour, for each seller
 * Maximum: Every 15 minutes, for each seller
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.data
 * @param {number} request.data.shop - Shop ID
 * @param {Object} request.data.file - product info in file
 * @returns {Promise} Promise object
 */
const importProduct = ({ url, headers, data }) => {
  const { error, value } = importProductSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  const bodyFormData = new FormData();
  bodyFormData.append('shop', value.shop);
  bodyFormData.append('file', value.file);
  return client.postRequest({
    url: url + PRODUCT_IMPORT_PATH,
    headers: {
      ...headers,
      'Content-Type': 'multipart/form-data',
      ...bodyFormData.getHeaders(),
    },
    data: bodyFormData,
  });
};

/**
 * Get the import status for a product import [GET P42]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#P42}
 * Recommended: Once per minute until getting the import final status
 * Maximum: Once per minute
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {number} request.importId
 * @returns {Promise} Promise object
 */
const getStatusForProductImportByImportId = ({ url, headers, importId }) => {
  const { error, value } = Joi.number().positive().validate(importId);
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + PRODUCT_IMPORT_STATUS_PATH(value),
    headers,
  });
};

/**
 * Export products [CM51]{@link https://help.mirakl.net/help/api-doc/front/mcm.html#CM51}
 * Delta export of the MCM products that are accepted and valid in CSV or JSON format.
 * To export MCM products, use the following headers:
 *  - accept: text/csv for a CSV format export
 *  - accept: application/json, by default export format is JSON.
 * Define a cron job in your system that periodically calls this API and sets the
 * previous request date as the updated_since query parameter.
 * Your integration logic must rely on header codes and not on column numbers.
 * Headers are subject to change depending on the exported products.
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.params
 * @param {string} request.params.updated_since
 * @param {string} request.params.updated_to
 * @param {string[]} request.params.acceptance_status
 * @param {string[]} request.params.validation_status
 * @param {string[]} request.params.synchronization_status
 * @param {string[]} request.params.product_sku
 * @param {string[]} request.params.product_id
 * @param {string[]} request.params.variant_group_code
 * @param {string[]} request.params.catalog
 * @param {string[]} request.params.rejection_reason
 * @param {string[]} request.params.category
 */
const exportProducts = ({ url, headers, params }) => {
  const { error, value } = exportProductsQueryParamSchema.validate(params);
  if (error) {
    return validationError(error.details);
  }
  // eslint-disable-next-line no-restricted-syntax
  for (const [key, val] of Object.entries(value)) {
    if (Array.isArray(val)) {
      value[key] = transformArrayToString(val);
    }
  }
  return client.getRequest({
    url: url + PRODUCT_EXPORT_PATH,
    headers,
    params: value,
  });
};

/**
 * Export attributes [PM11]{@link https://help.mirakl.net/help/api-doc/front/mmp.html#PM11}
 * Delta export of the MMP attributes that are accepted and valid in JSON format.
 * To export MCM attributes, use the following headers:
 *  - accept: application/json, by default export format is JSON.
 * Get the product attribute configuration
 * Retrieves all attributes for parents and children of the requested hierarchy
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.params
 * @param {string} request.params.hierarchy
 * @param {Number} request.params.max_level
 * @param {Boolean} request.params.with_roles
 */
const exportAttributes = ({ url, headers, params }) => {
  const { error, value } = exportAttributesSchema.validate(params);
  if (error) {
    return validationError(error.details);
  }
  // eslint-disable-next-line no-restricted-syntax
  for (const [key, val] of Object.entries(value)) {
    if (Array.isArray(val)) {
      value[key] = transformArrayToString(val);
    }
  }
  return client.getRequest({
    url: url + PRODUCT_ATTRIBUTE_PATH,
    headers,
    params: value,
  });
};

/**
 * Export value list [VL11]{@link https://help.mirakl.net/help/api-doc/front/mmp.html#VL11}
 * Delta export of the MMP VALUE_LIST that are accepted and valid in JSON format.
 * To export MCM value list, use the following headers:
 *  - accept: application/json, by default export format is JSON.
 * Get information about operator's value lists
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.params
 * @param {string} request.params.code
 */
const exportValueList = ({ url, headers, params }) => {
  const { error, value } = exportValueListSchema.validate(params);
  if (error) {
    return validationError(error.details);
  }
  // eslint-disable-next-line no-restricted-syntax
  for (const [key, val] of Object.entries(value)) {
    if (Array.isArray(val)) {
      value[key] = transformArrayToString(val);
    }
  }
  return client.getRequest({
    url: url + VALUE_LIST_PATH,
    headers,
    params: value,
  });
};

/**
 * Synchronize products [CM21]{@link https://help.mirakl.net/help/api-doc/operator/mcm.html#CM21}
 * Synchronize products from your system to Mirakl Catalog Manager. Use this API to:
 *  - confirm integration of products previously exported: send the mirakl_product_id and your own product_sku
 *  - report integration errors of products previously exported: send the mirakl_product_id and the integration_errors values
 *  - create products: send your own product_sku, complete product data, and potential catalogs assigned to the product. Products are accepted by default.
 *  - update products: send your own product_sku, the mirakl_product_id, additional product data, potential catalogs assigned to the product and acceptance status. By default products are accepted.
 *  - accept or reject products: send an acceptance entry with the status ACCEPTED, REJECTED, CHANGES_REQUESTED or TO_REVIEW. If the status is REJECTED, CHANGES_REQUESTED or TO_REVIEW, add the reason_code and an optional message.
 *    When using the REJECTED status, you must use a reason for definitive rejections, and use a reason for temporary rejections for CHANGES_REQUESTED and TO_REVIEW statuses
 *  - delete products: send the mirakl_product_id and the operation DELETE
 * @param {string} url
 * @param {string} headers
 * @param {object} data
 * @returns {{data: *, isOk: boolean, message: string, status: *}|{data: *, isOk: boolean, message: string, status: *}|Promise<{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean}|{data: *, isOk: boolean}|undefined>}
 */
const syncProducts = ({ url, headers, data }) => {
  const { error, value } = syncProductsSchema.validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client.postRequest({
    url: url + MCM_PRODUCT_SYNC_PATH,
    headers,
    data: value
  });
};

/**
 * Export deleted products [CM61]{@link https://help.mirakl.net/help/api-doc/operator/mcm.html#CM61}
 * Delta export of the MCM deleted products in CSV or JSON format.
 * This API only exports products deleted since less than 1 year.
 * To export MCM deleted products, use the following headers:
 *  - accept: text/csv for a CSV format export
 *  - accept: application/json, by default export format is JSON.
 * @param {string} url
 * @param {object} headers
 * @param {object} [params]
 * @param {string} [params.deleted_from]
 * @param {string} [params.deleted_to]
 * @param {string[]} [params.product_id]
 * @returns {{data: *, isOk: boolean, message: string, status: *}|{data: *, isOk: boolean, message: string, status: *}|Promise<{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean}|{data: *, isOk: boolean}|undefined>}
 */
const exportDeletedProducts = ({ url, headers, params }) => {
  const { error, value } = exportDeletedProductsQueryParamSchema.validate(params);
  if (error) {
    return validationError(error.details);
  }
  // eslint-disable-next-line no-restricted-syntax
  for (const [key, val] of Object.entries(value)) {
    if (Array.isArray(val)) {
      value[key] = transformArrayToString(val);
    }
  }
  return client.getRequest({
    url: url + DELETED_PRODUCT_EXPORT_PATH,
    headers,
    params: value
  });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient' || name === 'FrontClient') {
      ctx.prototype.exportProducts = function (params) {
        return exportProducts({ ...this._config, params });
      };
      ctx.prototype.importProduct = function (data) {
        return importProduct({ ...this._config, data });
      };
      ctx.prototype.getStatusForProductImportByImportId = function (
        importId
      ) {
        return getStatusForProductImportByImportId({
          ...this._config,
          importId,
        });
      };
      ctx.prototype.exportAttributes = function (params) {
        return exportAttributes({ ...this._config, params });
      };
      ctx.prototype.exportValueList = function (params) {
        return exportValueList({ ...this._config, params });
      };
      ctx.prototype.syncProducts = function (data) {
        return syncProducts({ ...this._config, data });
      };
      ctx.prototype.exportDeletedProducts = function (params) {
        return exportDeletedProducts({ ...this._config, params });
      };
    }
  }
};
