const Joi = require('joi');
const client = require('../core/client');
const { validationError } = require('../core/result');

const PROMOTIONS_PATH = '/api/promotions';
const EXPORT_PROMOTIONS = `${PROMOTIONS_PATH}/offers`;

/**
 * Get a CSV file listing the promotions associated with offers [GET PR51]{@link https://onceitnz-dev.mirakl.net/help/api-doc/front/mmp.html#PR51}
 * Returns a binary Csv diff file with
 * offer-id;shop-id;trigger-promotion-ids;reward-promotion-ids
 * Offers are sorted by id in ascending order
 * Recommended: Once per day
 * Maximum: Once per hour
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {string} [request.last_request_date] - date-time format
 */
const exportPromotions = ({
  url,
  headers,
  last_request_date,
}) => {
  const { error, value } = Joi.object()
    .keys({
      last_request_date: Joi.date().iso().optional(),
    })
    .optional()
    .validate({ last_request_date });
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: url + EXPORT_PROMOTIONS,
    headers,
    params: value,
  });
};

/**
 * Get details of a promotion [GET PR02] {@link https://help.mirakl.net/help/api-doc/operator/mmp.html#PR02}
 * Get a promotion details, including a list of the active offers associated with this promotion
 * Note: this resource supports locale parameter (see documentation)
 * Note: this resource supports offset pagination (see documentation)
 * Recommended: Each time an offer page must display a promotion
 * Maximum: Each time an offer page must display a promotion
 * @param url
 * @param headers
 * @param shopId
 * @param promotionInternalId
 * @returns {Promise<{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean}|undefined>|{data: *, isOk: boolean, message: string, status: *}}
 */
const getPromotionDetails = ({
  url, headers, shopId, promotionInternalId
}) => {
  const { error } = Joi.object()
    .keys({
      shopId: Joi.number().required(),
      promotionInternalId: Joi.string().required()
    })
    .optional()
    .validate({ shopId, promotionInternalId });
  if (error) {
    return validationError(error.details);
  }
  return client.getRequest({
    url: `${url + PROMOTIONS_PATH}/${shopId}/${promotionInternalId}`,
    headers,
  });
};

/**
 * PR01 request schema
 * @type {Joi.ObjectSchema<any>}
 */
const getListPromotionsSchema = Joi.object().keys({
  ids: Joi.array().items(Joi.object().keys({
    shopId: Joi.string().required(),
    promotionId: Joi.string().required()
  })).optional(),
  types: Joi.array().items(Joi.string().valid('AMOUNT_OFF', 'FREE_ITEMS', 'PERCENTAGE_OFF').required()).optional(),
  states: Joi.array().items(Joi.string().valid('PENDING', 'ACTIVE', 'EXPIRED').required()).optional(),
  shop_ids: Joi.array().items(Joi.string().required()).optional(),
  ending_after: Joi.string().isoDate().optional(),
  starting_before: Joi.string().isoDate().optional(),
  date_created_from: Joi.string().isoDate().optional(),
  date_created_to: Joi.string().isoDate().optional(),
  trigger_offer_ids: Joi.string().optional(),
  reward_offer_ids: Joi.string().optional(),
  channel_codes: Joi.string().optional(),
  max: Joi.number().positive().max(100).optional()
    .default(10),
  offset: Joi.number().min(0).optional().default(0),
}).optional();

/**
 * List promotions [GET PR01] {@link https://help.mirakl.net/help/api-doc/operator/mmp.html#PR01}
 * Note: this resource supports offset pagination (see documentation)
 * Recommended: At each promotion listing or promotion details page display
 * Maximum: At each promotion listing or promotion details page display
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object[]} [ids]
 * @param {string[]} [types]
 * @param {string[]} [states]
 * @param {string[]} [shop_ids]
 * @param {string} [ending_after]
 * @param {string} [starting_before]
 * @param {string} [date_created_from]
 * @param {string} [date_created_to]
 * @param {string} [trigger_offer_ids]
 * @param {string} [reward_offer_ids]
 * @param {string} [channel_codes]
 * @param {number} [max]
 * @param {number} [offset]
 * @returns {Promise<{data: *, isOk: boolean, message: *, status: *}|{data: *, isOk: boolean}|undefined>|{data: *, isOk: boolean, message: string, status: *}}
 */
const getListPromotions = ({
  url, headers, ids, types, states, shop_ids, ending_after, starting_before, date_created_from, date_created_to, trigger_offer_ids, reward_offer_ids, channel_codes, max, offset
}) => {
  const { error, value } = getListPromotionsSchema.validate({
    ids, types, states, shop_ids, ending_after, starting_before, date_created_from, date_created_to, trigger_offer_ids, reward_offer_ids, channel_codes, max, offset
  });
  if (error) {
    return validationError(error.details);
  }
  const params = {
    ...value,
    ids: value.ids && value.ids.map((i) => `${i.shopId}|${i.promotionId}`).join(','),
    types: value.types && value.types.join(','),
    states: value.states && value.states.join(','),
    shop_ids: value.shop_ids && value.shop_ids.join(','),
  };
  return client.getRequest({
    url: `${url + PROMOTIONS_PATH}`,
    headers,
    params
  });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient') {
      ctx.prototype.exportPromotions = function (params) {
        return exportPromotions({ ...this._config, ...params });
      };
      ctx.prototype.getPromotionDetails = function (params) {
        return getPromotionDetails({ ...this._config, ...params });
      };
      ctx.prototype.getListPromotions = function (params) {
        return getListPromotions({ ...this._config, ...params });
      };
    }
  }
};
