const Joi = require('joi');
const client = require('../core/client');
const { validationError, failure } = require('../core/result');
const { getCountryISOCodes, getLocaleShortCodes } = require('../core/utils');

const SHIPPING_PATH = '/api/shipping';
const SHIPPING_RATES_PATH = `${SHIPPING_PATH}/rates`;

const conditionalByComputeOrderTaxes = Joi.string().when(
  'compute_order_taxes',
  {
    is: true,
    then: Joi.required(),
    otherwise: Joi.optional(),
  }
);

const shippingRateQuerySchema = Joi.object().keys({
  offers: Joi.array().items(
    Joi.object().keys({
      id: Joi.number().positive().required(),
      qty: Joi.number().positive().required(),
      shippingTypeCode: Joi.string().optional(),
      leadTimeToShip: Joi.number().positive().optional(),
    })
  ),
  shipping_zone_code: Joi.string().required(),
  pricing_channel_code: Joi.string().optional(),
  compute_order_taxes: Joi.boolean().optional().default(false),
  customer_shipping_address_street_1: conditionalByComputeOrderTaxes,
  customer_shipping_address_street_2: Joi.string().optional(),
  customer_shipping_address_state: conditionalByComputeOrderTaxes,
  customer_shipping_address_zipcode: conditionalByComputeOrderTaxes,
  customer_shipping_address_city: conditionalByComputeOrderTaxes,
  customer_shipping_address_country_iso_code: conditionalByComputeOrderTaxes.valid(
    ...getCountryISOCodes
  ),
  customer_code: Joi.string().optional(),
  locale: Joi.string()
    .valid(...getLocaleShortCodes)
    .optional(),
});

/**
 * List shipping charges of each offer, grouped by orders [GET SH02]{@link https://help.mirakl.net/help/api-doc/front/mmp.html#SH02}
 * This method returns the ordered quantity next to the available quantity of each offer.
 * The client should check if requested offer quantity is available. If offer supply is insufficient, Mirakl won't block the order, the client is responsible for handling that scenario.
 * The offer available quantity is:
 *  - 0: if quantity = 0 or the offer is not available
 *  - x: quantity available
 * Result is sort by shop name then by index for shipping types and by identifier for offers
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.params
 * @param {string} request.params.offers
 * @param {string} request.params.shipping_zone_code
 * @param {string} [request.params.pricing_channel_code]
 * @param {boolean} [request.params.compute_order_taxes]
 * @param {string} [request.params.customer_shipping_address_street_1]
 * @param {string} [request.params.customer_shipping_address_street_2]
 * @param {string} [request.params.customer_shipping_address_state]
 * @param {string} [request.params.customer_shipping_address_zipcode]
 * @param {string} [request.params.customer_shipping_address_city]
 * @param {string} [request.params.customer_shipping_address_country_iso_code]
 * @param {string} [request.params.customer_code]
 * @param {string} [request.params.locale]
 */
const getListShippingRates = ({ url, headers, params }) => {
  const { error, value } = shippingRateQuerySchema.validate(params);
  if (error) {
    return validationError(error.details);
  }
  const newParams = {
    ...value,
    offers: value.offers
      .map((offer) => Object.values(offer)
        .filter((v) => v)
        .join('|'))
      .join(','),
  };
  return client
    .getRequest({
      url: url + SHIPPING_RATES_PATH,
      headers,
      params: newParams,
    })
    .then((res) => {
      if (res.isOk && res.data.errors.length) {
        return failure(res.data.errors[0].error_code, res.data);
      }
      return res;
    });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'FrontClient') {
      ctx.prototype.getListShippingRates = function (params) {
        return getListShippingRates({ ...this._config, params });
      };
    }
  }
};
