const Joi = require('joi').extend(require('joi-currency-code'));
const client = require('../core/client');
const { failure, validationError } = require('../core/result');

const { getCountryISOCodes } = require('../core/utils');

const SHOP_PATH = '/api/shops';
const SHOP_INFO = (id) => `${SHOP_PATH}/${id}`;
const OFFER_PATH = (id) => `${SHOP_INFO(id)}/offers`;
const LIST_EVALUATIONS_PATH = (id) => `${SHOP_INFO(id)}/evaluations`;
const LIST_DOCUMENTS = `${SHOP_PATH}/documents`;

const resolveShopResponse = (res) => {
  if (res.isOk) {
    const errors = res.data.shop_returns
      .filter((obj) => obj.shop_error)
      .map((obj) => obj.shop_error)
      .flatMap((obj) => obj.errors);
    if (errors && errors.length > 0) {
      return failure(errors[0].code, res.data);
    }
  }
  return res;
};

const locale = Joi.string().optional();

const paginationSchema = {
  max: Joi.number().positive().max(100).optional()
    .default(10),
  offset: Joi.number().min(0).optional().default(0),
  sort: Joi.string().optional(),
  order: Joi.string().valid('asc', 'desc').optional().default('asc'),
};

const shopSchema = Joi.object().keys({
  address: Joi.object({
    city: Joi.string().max(255).required(),
    civility: Joi.string().max(20).required(),
    country: Joi.string().max(255).required(),
    firstname: Joi.string().max(255).required(),
    lastname: Joi.string().max(255).required(),
    phone: Joi.string().max(50).optional().allow(null, '')
      .default(''),
    phone_secondary: Joi.string()
      .max(50)
      .optional()
      .allow(null, '')
      .default(''),
    state: Joi.string().optional().allow(null, '').default(''),
    street1: Joi.string().required(),
    street2: Joi.string().optional().allow(null, '').default(''),
    zip_code: Joi.string().max(255).optional(),
  }),
  channel_codes: Joi.array().max(36).items(Joi.string()).optional(),
  closed_from_date: Joi.date().iso().optional(),
  closed_to_date: Joi.date().iso().optional(),
  currency_iso_code: Joi.string().currency().optional(),
  description: Joi.string().max(3000).optional().allow(null, '')
    .default(''),
  domains: Joi.array()
    .items(Joi.string().valid('PRODUCT', 'SERVICE'))
    .optional(),
  email: Joi.string().max(255).email().required(),
  fax: Joi.string().max(20).optional().allow(null, '')
    .default(''),
  immunity_date: Joi.date().iso().optional(),
  is_professional: Joi.boolean().optional(),
  existing_user: Joi.object({
    email: Joi.string().max(255).email().required(),
    password: Joi.string().max(255).optional(),
  }).optional(),
  new_user: Joi.object({
    email: Joi.string().max(255).email().required(),
    locale: Joi.string().max(255).optional(),
    password: Joi.string().min(8).max(255).optional(),
  }).optional(),
  operator_internal_id: Joi.string()
    .max(255)
    .optional()
    .allow(null, '')
    .default(''),
  password: Joi.string().min(8).max(255).optional(),
  payment_blocked: Joi.boolean().required(),
  payment_info: Joi.object({
    owner: Joi.string().required(),
    '@type': Joi.string(),
    bank_account_number: Joi.string().required(),
    bank_city: Joi.string(),
    bank_name: Joi.string(),
    bank_street: Joi.string(),
    bank_zip: Joi.string(),
    bic: Joi.string(),
    routing_number: Joi.string(),
  })
    .optional()
    .allow(null),
  pro_details: Joi.object({
    corporate_name: Joi.string().max(255).required(),
    identification_number: Joi.string().max(255).required(),
    tax_identification_number: Joi.string()
      .max(255)
      .optional()
      .allow(null, '')
      .default(''),
  }).optional(),
  return_policy: Joi.string().max(5000).optional().allow(null, '')
    .default(''),
  shipping_country: Joi.string()
    .valid(...getCountryISOCodes)
    .optional(),
  shop_additional_fields: Joi.array()
    .items(
      Joi.object({
        code: Joi.string().required(),
        value: Joi.any().required().allow(''),
      })
    )
    .optional()
    .allow(null),
  shop_name: Joi.string().max(255).required(),
  suspend: Joi.boolean().required(),
  web_site: Joi.string().max(255).optional().allow(null, '')
    .default(''),
});

const shopQueryParamSchema = Joi.object().keys({
  shop_ids: Joi.array().max(100).items(Joi.string()).optional(),
  premium: Joi.string()
    .valid('TRUE', 'FALSE', 'ALL')
    .empty(['', null])
    .default('ALL'),
  state: Joi.string().valid('PEN', 'CLOSE', 'SUSPENDED').optional(),
  updated_since: Joi.date().iso().optional(),
  domains: Joi.array()
    .items(Joi.string())
    .has(Joi.string().valid('PRODUCT', 'SERVICE').required())
    .optional(),
  kyc_status: Joi.array()
    .items(Joi.string())
    .has(
      Joi.string()
        .valid('PENDING_SUBMISSION', 'PENDING_APPROVAL', 'REFUSED', 'APPROVED')
        .required()
    )
    .optional(),
  paginate: Joi.boolean().empty([null]).optional().default(true),
});

const offerQueryParamSchema = Joi.object()
  .keys({
    offer_state_codes: Joi.array().items(Joi.string().required()).optional(),
    all_offers: Joi.boolean().optional().default(false),
    product_skus: Joi.array().items(Joi.string().required()).optional(),
    favorite: Joi.boolean().optional().default(false),
    pricing_channel_code: Joi.string().optional(),
    locale,
  })
  .optional();

const listShopDocumentParamSchema = Joi.object().keys({
  shop_ids: Joi.array().items(Joi.string(), Joi.number()).min(1).optional(),
  include_deleted: Joi.boolean().optional(),
  updated_since: Joi.date().iso().optional(),
});

/**
 * Create shops [POST S06]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#S06}
 * A maximum of 100 shops can be sent at once.
 * Recommended: At each store automatic creation from your CRM system
 * Maximum: Once per minute, 100 stores per call
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.data [request payload]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#S06}
 */
const createShops = ({ url, headers, data }) => {
  const { error, value } = Joi.object({
    shops: Joi.array().items(shopSchema),
  }).validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client
    .postRequest({
      url: url + SHOP_PATH,
      headers,
      data: value,
    })
    .then(resolveShopResponse);
};

/**
 * Patch update shops [PUT S07]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#S07}
 * Update shops information field by field: unspecified fields will not be updated.
 * To remove the value for a field, send it with the 'null'
 * value (not allowed for the required fields).
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.data [request payload]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#S07}
 */
const updateShops = ({ url, headers, data }) => {
  const { error, value } = Joi.object({
    shops: Joi.array().items(
      shopSchema.append({ shop_id: Joi.number().integer().required() })
    ),
  }).validate(data);
  if (error) {
    return validationError(error.details);
  }
  return client
    .putRequest({
      url: url + SHOP_PATH,
      headers,
      data: value,
    })
    .then(resolveShopResponse);
};

/**
 * List all shops [GET S20]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#S20}
 * Note: this resource supports offset pagination [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#offset-pagination}
 * Recommended:
 * - Differential: every 15 minutes
 * - Full: once per day
 * Maximum:
 * - Differential: every 5 minutes
 * - Full: every 4 hours
 * @param {Object} request
 * @param {string} request.url
 * @param {Object} request.headers
 * @param {Object} request.params
 * @param {string} [request.params.shop_ids]
 * @param {string} [request.params.premium]
 * @param {string} [request.params.state]
 * @param {string} [request.params.updated_since]
 * @param {string} [request.params.domains]
 * @param {string[]} [request.params.kyc_status]
 * @param {boolean} [request.params.paginate]
 * @param {number} [request.params.offset]
 * @param {number} [request.params.max]
 * @param {string} [request.params.sort]
 * @param {string} [request.params.order]
 */
const getListAllShops = ({ url, headers, params }) => {
  const result = shopQueryParamSchema.keys(paginationSchema).validate(params);
  let { value } = result;
  const { error } = result;
  if (error) {
    return validationError(error.details);
  }

  if (value && value.shop_ids) {
    if (value.shop_ids) {
      value = { ...value, shop_ids: value.shop_ids.join(',') };
    }
    if (value.domains) {
      value = { ...value, domains: value.domains.join(',') };
    }
  }

  return client.getRequest({
    url: url + SHOP_PATH,
    headers,
    params: value,
  });
};

/**
 * List offers of a shop [GET S04]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#S04}
 * Note: this resource supports locale parameter [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#locale}
 * Note: this resource supports offset pagination [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#offset-pagination}
 * Recommended: On each store page that includes a list of offers
 * Maximum: On each store page that includes a list of offers
 * @param {Object}  request
 * @param {string}  request.url
 * @param {Object}  request.headers
 * @param {number}  request.shopId
 * @param {Object}  [request.params]
 * @param {string}  [request.params.offer_state_codes]
 * @param {boolean} [request.params.all_offers]
 * @param {string}  [request.params.product_skus]
 * @param {boolean} [request.params.favorite]
 * @param {string}  [request.params.pricing_channel_code]
 * @param {number}  [request.params.offset]
 * @param {number}  [request.params.max]
 * @param {string}  [request.params.sort] - totalPrice(default),price, and productTitle
 * @param {string}  [request.params.order]
 * @param {string}  [request.locale]
 */
const getListOffersByShopId = ({
  url, headers, shopId, params
}) => {
  const validatedShopId = Joi.number().integer().required().validate(shopId);
  if (validatedShopId.error) {
    return validationError(validatedShopId.error.details);
  }
  const validatedParams = offerQueryParamSchema
    .keys(paginationSchema)
    .validate(params);
  if (validatedParams.error) {
    return validationError(validatedParams.error.details);
  }
  return client.getRequest({
    url: url + OFFER_PATH(validatedShopId.value),
    params: validatedParams.value,
    headers,
  });
};

/**
 * List evaluations of a shop [GET S03]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#S03}
 * Note: this resource supports locale parameter [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#locale}
 * Note: this resource supports offset pagination [(see documentation)]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#offset-pagination}
 * Recommended: On each shop evaluations page view
 * Maximum: On each shop evaluations page view
 * @param {Object}  request
 * @param {string}  request.url
 * @param {Object}  request.headers
 * @param {number}  request.shopId
 * @param {Object}  [request.params]
 * @param {number}  [request.params.offset]
 * @param {number}  [request.params.max]
 * @param {string}  [request.params.sort] - date(default),and grade
 * @param {string}  [request.params.order]
 * @param {string}  [request.locale]
 */
const getListEvaluationsByShopId = ({
  url, headers, shopId, params
}) => {
  const validatedShopId = Joi.number().integer().required().validate(shopId);
  if (validatedShopId.error) {
    return validationError(validatedShopId.error.details);
  }
  const validatedParams = Joi.object(paginationSchema)
    .keys({ locale })
    .optional()
    .validate(params);
  if (validatedParams.error) {
    return validationError(validatedParams.error.details);
  }
  return client.getRequest({
    url: url + LIST_EVALUATIONS_PATH(validatedShopId.value),
    params: validatedParams.value,
    headers,
  });
};

/**
 * List shop's documents [GET S30]{@link https://help.mirakl.net/help/api-doc/operator/mmp.html#S30}
 * You must specify at least one of the following parameters: shop_ids, updated_since
 * Recommended: At each display of a page that includes documents from a store (for example: during the KYC process)
 * Maximum: At each display of a page that includes documents from a store (for example: during the KYC process)
 * @param {Object}    request
 * @param {string}    request.url
 * @param {Object}    request.headers
 * @param {number[]}  request.shop_ids
 * @param {boolean}   request.include_deleted
 * @param {string}    request.updated_since
 */
const getListShopDocuments = ({
  url, headers, shop_ids, include_deleted, updated_since
}) => {
  const { error, value } = listShopDocumentParamSchema.validate({ shop_ids, include_deleted, updated_since });
  if (error) {
    return validationError(error.details);
  }
  const params = {
    ...value
  };
  if (value.shop_ids && value.shop_ids.length > 0) {
    params.shop_ids = value.shop_ids.join(',');
  }
  return client.getRequest({
    url: url + LIST_DOCUMENTS,
    params,
    headers
  });
};

module.exports = (ctx) => {
  if (ctx) {
    const { name } = ctx;
    if (name === 'OperatorClient') {
      ctx.prototype.createShops = function (data) {
        return createShops({ ...this._config, data });
      };
      ctx.prototype.updateShops = function (data) {
        return updateShops({ ...this._config, data });
      };
      ctx.prototype.getListAllShops = function (params) {
        return getListAllShops({ ...this._config, params });
      };
      ctx.prototype.getListOffersByShopId = function (shopId, params) {
        return getListOffersByShopId({ ...this._config, shopId, params });
      };
      ctx.prototype.getListEvaluationsByShopId = function (shopId, params) {
        return getListEvaluationsByShopId({ ...this._config, shopId, params });
      };
      ctx.prototype.getListShopDocuments = function ({ shop_ids, include_deleted, updated_since }) {
        return getListShopDocuments({
          ...this._config, shop_ids, include_deleted, updated_since
        });
      };
    }
  }
};
